# -*- coding: utf-8 -*-

"""Integration test cases.
"""

from django.core.urlresolvers import reverse
from django.test import TestCase

from testproj.app import views

from djangoflash.context_processors import CONTEXT_VAR
from djangoflash.middleware import _SESSION_KEY, FlashScope


class IntegrationTestCase(TestCase):
    """Test the middleware and the context processors working within a real
    Django application.
    """
    def _flash(self):
        """Shortcut to get the flash scope from the view context.
        """
        return self.response.context[CONTEXT_VAR]
    
    def test_session_state_for_unused_flash(self):
        """Flash scope shouldn't be stored in the session if there's no flash.
        """
        self.response = self.client.get(reverse(views.render_template))
        self.assertFalse(_SESSION_KEY in self.client.session)
    
    def test_session_state_for_used_flash(self):
        """Flash scope should be removed from the session if there's no flash.
        """
        self.response = self.client.get(reverse(views.set_flash_var))
        self.response = self.client.get(reverse(views.render_template))
        self.assertTrue(_SESSION_KEY in self.client.session)
        
        # Flash scope should be removed from the session
        self.response = self.client.get(reverse(views.render_template))
        self.assertFalse(_SESSION_KEY in self.client.session)
    
    def test_default_lifecycle(self):
        """A value should be automatically removed from the flash scope.
        """
        self.response = self.client.get(reverse(views.set_flash_var))
        self.assertEqual('Message', self._flash()['message'])
        
        self.response = self.client.get(reverse(views.render_template))
        self.assertEqual('Message', self._flash()['message'])
        
        # Flash value will be removed when this request hits the app
        self.response = self.client.get(reverse(views.render_template))
        self.assertFalse('message' in self._flash())
    
    def test_keep_lifecycle(self):
        """A value shouldn't be removed from the flash scope when it is kept.
        """
        self.response = self.client.get(reverse(views.set_flash_var))
        self.assertEqual('Message', self._flash()['message'])
        
        self.response = self.client.get(reverse(views.keep_var))
        self.assertEqual('Message', self._flash()['message'])
        
        # Flash value won't be removed now because it was explicitely kept
        self.response = self.client.get(reverse(views.render_template))
        self.assertEqual('Message', self._flash()['message'])
        
        # Flash value will be removed when this request hits the app
        self.response = self.client.get(reverse(views.render_template))
        self.assertFalse('message' in self._flash())
    
    def test_now_lifecycle(self):
        """An immediate value shouldn't survive the next request.
        """
        self.response = self.client.get(reverse(views.set_now_var))
        self.assertEqual('Message', self._flash()['message'])
        
        # Flash value will be removed when this request hits the app
        self.response = self.client.get(reverse(views.render_template))
        self.assertFalse('message' in self._flash())
    
    def test_discard_lifecycle(self):
        """A discarded value shouldn't survive to the next request.
        """
        self.response = self.client.get(reverse(views.discard_var))
        self.assertEqual('Message', self._flash()['message'])
        
        # Flash value will be removed when this request hits the app
        self.response = self.client.get(reverse(views.render_template))
        self.assertFalse('message' in self._flash())
    
    def test_multiple_variables_lifecycle(self):
        """The flash scope should control several values independently.
        """
        self.response = self.client.get(reverse(views.set_flash_var))
        self.assertEqual('Message', self._flash()['message'])
        
        self.response = self.client.get(reverse(views.set_another_flash_var))
        self.assertEqual('Message', self._flash()['message'])
        self.assertEqual('Another message', self._flash()['anotherMessage'])
        
        # 'message' will be removed when this request hits the app
        self.response = self.client.get(reverse(views.render_template))
        self.assertFalse('message' in self._flash())
        self.assertEqual('Another message', self._flash()['anotherMessage'])
        
        # 'anotherMessage' will be removed when this request hits the app
        self.response = self.client.get(reverse(views.render_template))
        self.assertFalse('message' in self._flash())
        self.assertFalse('anotherMessage' in self._flash())
    
    def test_replace_flash_scope(self):
        """The flash scope should be an instance of FlashScope.
        """
        request = lambda: self.client.get(reverse(views.replace_flash))
        self.assertRaises(TypeError, request)



# -*- coding: utf-8 -*-

"""djangoflash.models test cases.
"""

from unittest import TestCase

from djangoflash.middleware import FlashScope

class FlashScopeTestCase(TestCase):
    """Test the FlashScope object.
    """
    def setUp(self):
        """Create a FlashScope object to be used by the test methods.
        """
        self.scope = FlashScope()
    
    def test_contains(self):
        """"key in flash" syntax should be supported.
        """
        self.assertFalse('message' in self.scope)
        self.scope['message'] = 'Message'
        self.assertTrue('message' in self.scope)
    
    def test_get_item(self):
        """"flash[key]" syntax should be supported.
        """
        get_item = lambda: self.scope['message']
        self.assertRaises(KeyError, get_item);
        self.scope['message'] = 'Message'
        self.assertEqual('Message', get_item());
        
    
    def test_set_item(self):
        """"flash[key] = value" syntax should be supported.
        """
        self.scope['message'] = 'Message'
        self.assertEqual('Message', self.scope['message']);
    
    def test_del_item(self):
        """"del flash[key]" syntax should be supported.
        """
        self.scope['message'] = 'Message'
        del self.scope['message']
        self.assertFalse('message' in self.scope)
    
    def test_clear(self):
        """flash.clear() should remove all items from the flash scope.
        """
        self.scope['message'] = 'Message'
        self.scope['anotherMessage'] = 'Another message'
        self.scope.clear()
        self.assertEqual(0, len(self.scope))
    
    def test_len(self):
        """"len(flash)" syntax should be supported.
        """
        self.scope['message'] = 'Message'
        self.assertEqual(1, len(self.scope))
    
    def test_keys(self):
        """Should return the list of keys stored in the flash scope.
        """
        self.scope['message'] = 'Message'
        self.assertEqual(['message'], self.scope.keys())
    
    def test_values(self):
        """Should return the list of values stored in the flash scope.
        """
        self.scope['message'] = 'Message'
        self.assertEqual(['Message'], self.scope.values())
    
    def test_items(self):
        """Should return the list of items stored in the flash scope.
        """
        self.scope['message'] = 'Message'
        self.assertEqual([('message', 'Message')], self.scope.items())
    
    def test_iterkeys(self):
        """Should return an iterator to the keys stored in the flash scope.
        """
        self.scope['message'] = 'Message'
        iterator = self.scope.iterkeys()
        self.assertEqual('message', iterator.next())
        self.assertRaises(StopIteration, iterator.next)
    
    def test_itervalues(self):
        """Should return an iterator to the values stored in the flash scope.
        """
        self.scope['message'] = 'Message'
        iterator = self.scope.itervalues()
        self.assertEqual('Message', iterator.next())
        self.assertRaises(StopIteration, iterator.next)
    
    def test_iteritems(self):
        """Should return an iterator to the items stored in the flash scope.
        """
        self.scope['message'] = 'Message'
        iterator = self.scope.iteritems()
        self.assertEqual(('message', 'Message'), iterator.next())
        self.assertRaises(StopIteration, iterator.next)
    
    def test_get(self):
        """Should return a default value if the given key doesn' exists.
        """
        self.assertEqual('Something', self.scope.get('message', 'Something'))
        self.scope['message'] = 'Message'
        self.assertEqual('Message', self.scope.get('message', 'Something'))
        self.assertEqual(None, self.scope.get('another_message'))
    
    def test_pop(self):
        """Should pop an item from the flash scope.
        """
        self.assertEqual(None,self.scope.pop('message'))
        self.scope['message'] = 'Message'
        self.assertEqual('Message', self.scope.pop('message'))
        self.assertFalse('message' in self.scope)
    
    def test_pop_used_value(self):
        """Should pop an used item from the flash scope.
        """
        self.scope['message'] = 'Message'
        self.scope.update()
        self.assertEqual('Message', self.scope.pop('message'))
        self.assertFalse('message' in self.scope)
    
    def test_has_key(self):
        """Should check if there's a value related with the given key.
        """
        self.assertFalse(self.scope.has_key('message'))
        self.scope['message'] = 'Message'
        self.assertTrue(self.scope.has_key('message'))
    
    def test_put(self):
        """Should put several values into the flash scope at the same time.
        """
        self.scope.put(msg1='Message1', msg2='Message2')
        self.assertEqual('Message1', self.scope['msg1'])
        self.assertEqual('Message2', self.scope['msg2'])
    
    def test_discard(self):
        """Should mark a flash-scoped value for removal.
        """
        self.scope['message'] = 'Message'
        self.scope.discard()
        
        self.scope.update()
        self.assertFalse('message' in self.scope)
    
    def test_keep(self):
        """Should mark all flash-scoped value for removal.
        """
        self.scope['message'] = 'Message'
        self.scope.update()
        self.scope.keep('message')
        self.scope.update()
        self.assertEqual('Message', self.scope['message'])
    
    def test_keep_all(self):
        """Should avoid the removal of all flash-scoped values.
        """
        self.scope['message'] = 'Message'
        self.scope.update()
        self.scope.keep()
        self.scope.update()
        self.assertEqual('Message', self.scope['message'])
    
    def test_alternative_now(self):
        """Immediate values (flash.now) should be supported.
        """
        self.scope.now(message='Message')
        self.assertEqual('Message', self.scope['message'])
        self.scope.update()
        self.assertFalse('message' in self.scope)
    
    def test_now(self):
        """"flash.now[key] = value" syntax should be supported.
        """
        self.scope.now['message'] = 'Message'
        self.assertEqual('Message', self.scope['message'])
        self.scope.update()
        self.assertFalse('message' in self.scope)
    
    def test_now_get_item(self):
        """"flash.now[key]" syntax should be supported.
        """
        self.scope.now['message'] = 'Message'
        self.assertEqual('Message', self.scope.now['message'])
    
    def test_now_contains(self):
        """"key in flash.now" syntax should be supported.
        """
        self.assertFalse('message' in self.scope.now)
        self.scope.now['message'] = 'Message'
        self.assertTrue('message' in self.scope.now)



# -*- coding: utf-8 -*-

"""
    Django-flash
    ~~~~~~~~~~~~

    Rails-like flash scope support for Django.

    :copyright: 2008-2009, Destaquenet Technology Solutions.
    :license: LGPL v3.
"""

__version__ = '1.4.2'




# Create your views here.

from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext


def render_template(request):
    return render_to_response('simple.html', {}, \
        context_instance=RequestContext(request))

def set_flash_var(request):
    request.flash['message'] = 'Message'
    return render_template(request)

def set_another_flash_var(request):
    request.flash['anotherMessage'] = 'Another message'
    return render_template(request)

def set_now_var(request):
    request.flash.now['message'] = 'Message'
    return render_template(request)

def keep_var(request):
    request.flash.keep('message')
    return render_template(request)

def discard_var(request):
    # Should behave the same way 'flash.now' does
    request.flash['message'] = 'Message'
    request.flash.discard('message')
    return render_template(request)

def replace_flash(request):
    request.flash = "Replacing the flash with a string"
    return render_template(request)



# -*- coding: utf-8 -*-

"""
This module provides the :class:`FlashMiddleware` class, which manages the
flash context whenever a HTTP requests hits the server.

To plug this middleware to your Django project, add the following snippet to
the ``settings.py`` file::

    MIDDLEWARE_CLASSES = (
        'django.contrib.sessions.middleware.SessionMiddleware',
        'djangoflash.middleware.FlashMiddleware',
    )


You need to add the :class:`SessionMiddleware` because Django-flash relies on
user's session to store the contents of the flash scope.

.. warning::
   The :class:`FlashMiddleware` class must be declared after the
   :class:`SessionMiddleware` class.
"""

from djangoflash.models import FlashScope
from djangoflash.context_processors import CONTEXT_VAR


"""This value is used to identify the flash scope object inside the user's
session.
"""
_SESSION_KEY = '_djflash_app'


class FlashMiddleware(object):
    """This middleware puts/gets a :class:`djangoflash.models.FlashScope`
    object to/from the user's session, and triggers the expiration of old
    flash-scoped objects.
    
    .. note::
       This class is designed to be used by the Django framework itself.
    """
    
    @staticmethod
    def get_context_from_request(request):
        """Gets the :class:`FlashScope` object from the *request* and returns
        it. If it couldn't be found, the method returns a brand new
        :class:`FlashScope` object.
        """
        context = FlashScope()
        if hasattr(request, CONTEXT_VAR):
            context = getattr(request, CONTEXT_VAR)
            if not isinstance(context, FlashScope):
                raise TypeError('Invalid Flash scope object: %s' % \
                    repr(context))
        return context
    
    @staticmethod
    def get_context_from_session(request):
        """Gets the :class:`FlashScope` object from the user's session and
        :meth:`update` it. If this object couldn't be found, this method
        returns a brand new :class:`FlashScope` object.
        """ 
        context = FlashScope()
        if hasattr(request, 'session') and _SESSION_KEY in request.session:
            context = request.session[_SESSION_KEY]
            context.update()
        return context
    
    @staticmethod
    def process_request(request):
        """This method is called by the Django framework when a *request*
        hits the server.
        """
        setattr(request, CONTEXT_VAR, \
            FlashMiddleware.get_context_from_session(request))
    
    @staticmethod
    def process_response(request, response):
        """This method is called by the Django framework when a *response* is
        sent back to the user.
        """
        if hasattr(request, 'session'):
            context = FlashMiddleware.get_context_from_request(request)
            if len(context) > 0:
                request.session[_SESSION_KEY] = context
            elif _SESSION_KEY in request.session:
                del request.session[_SESSION_KEY]
        return response



# -*- coding: utf-8 -*-

"""
This module provides the context processors that exposes the
:class:`djangoflash.models.FlashScope` object to view's templates.

To plug this context processor to your Django project, add the following
snippet to the ``settings.py`` file::

    TEMPLATE_CONTEXT_PROCESSORS = (
        'djangoflash.context_processors.flash',
    )


Doing this your view templates will be able to access the flash scope
contents using the ``flash`` context variable.

.. warning::
   Your views should use the :class:`RequestContext` class to render the view
   templates, otherwise the ``flash`` variable (along with *all* other
   variables provided by other context processors) won't be available to you.
   Please read the
   `Django documentation <http://docs.djangoproject.com/en/dev/ref/templates/api/>`_
   for further instructions.

"""

CONTEXT_VAR = 'flash'

def flash(request):
    """This context processor gets the :class:`FlashScope` object from the
    current *request* and adds it to the template context:
    
    .. code-block:: html+django
    
       <html>
           <head></head>
           <body>
               request.flash['message'] = {{ flash.message }}
           </body>
       </html>
    
    """
    return {CONTEXT_VAR:request.flash}



from django.conf.urls.defaults import *

from testproj.app import views

urlpatterns = patterns('',
    (r'^default/$', views.render_template),
    (r'^set_flash_var/$', views.set_flash_var),
    (r'^set_another_flash_var/$', views.set_another_flash_var),
    (r'^set_now_var/$', views.set_now_var),
    (r'^keep_var/$', views.keep_var),
    (r'^discard_var/$', views.discard_var),
    (r'^replace_flash/$', views.replace_flash),
)



# -*- coding: utf-8 -*-

"""Project's test suite.
"""

import sys

# Add the Django test project to system path
from django.core.management import setup_environ
import djangoflash.tests.testproj.settings as project_settings
sys.path.insert(0, setup_environ(project_settings))

# Bootstrap Django test environment
from django.test.utils import setup_test_environment
from django.db import connection
setup_test_environment()
connection.creation.create_test_db()

# Finally, add the tests to this suite
from models import *
from testproj.app.tests import *



# -*- coding: utf-8 -*-

"""This module provides the :class:`FlashScope` class, which provides a simple
way to pass temporary objects between views.
"""

class FlashScope(object):
    """The purpose of this class is to implement the *flash* scope, which is
    a storage context that is a middle ground between the request context and
    the session context.
    
    More specifically, the *flash* scope provides a simple way to pass
    temporary objects from a view method to another (or to a view template).
    Anything you place in this scope will survive for the next request only,
    being automatically removed.
    
    The following operations are supported by :class:`FlashScope` instances:

    .. describe:: len(f)
    
       Returns the number of items in the flash scope *f*.

    .. describe:: f[key]
    
       Returns the item of *f* with key *key*.  Raises a :exc:`KeyError` if
       *key* is not in the flash scope.

    .. describe:: f[key] = value
    
       Sets ``f[key]`` to *value*.

    .. describe:: del f[key]
    
       Removes ``f[key]`` from *f*.  Raises a :exc:`KeyError` if *key* is not
       in the map.

    .. describe:: key in f
    
       Returns ``True`` if *f* has a key *key*, else ``False``.

    .. describe:: key not in f
    
       Equivalent to ``not key in f``.
    
    .. describe:: f.now[key] = value
    
       Sets ``f[key]`` to *value* and marks it as *used*.
    
    .. describe:: f.now(**items)
    
       Puts the given *items* into *f* and marks them as *used*.
    """
    
    def __init__(self):
        """Returns a new :class:`FlashScope` object.
        """
        self._session = {}
        self._used = {}
        self.now = _ImmediateFlashScopeAdapter(self)
    
    def __contains__(self, key):
        """Returns ``True`` if there's a value under the given *key*.
        """
        return key in self._session
    
    def __getitem__(self, key):
        """Retrieves a value. Raises a :exc:`KeyError` if *key* does
        not exists.
        """
        return self._session[key]
    
    def __setitem__(self, key, value):
        """Puts a *value* into this flash scope under the given *key*.
        """
        self._session[key] = value
        self._update_status(key, is_used=False)
    
    def __delitem__(self, key):
        """Removes the value under the given *key*.
        """
        if key in self:
            del self._session[key]
        if key in self._used:
            del self._used[key]
    
    def __len__(self):
        """Returns the number of values inside this flash scope.
        """
        return len(self._session)
    
    def _update_status(self, key=None, is_used=True):
        """Updates the status of a given value (or all values if no *key*
        is given). The *is_used* argument tells if that value should be
        marked as *used* (should be discarded) or *unused* (should be kept).
        
        If a *used* value is being marked as *used* again, it is automatically
        removed from this flash scope.
        """
        if not key:
            for existing_key in self.keys():
                self._update_status(existing_key, is_used)
        else:
            if not is_used:
                if key in self._used:
                    del self._used[key]
            else:
                if key in self._used:
                    del self[key]
                else:
                    self._used[key] = None
    
    def keys(self):
        """Returns the list of keys.
        """
        return self._session.keys()
    
    def values(self):
        """Returns the list of values.
        """
        return self._session.values()
    
    def items(self):
        """Returns the list of items as tuples ``(key, value)``.
        """
        return self._session.items()
    
    def iterkeys(self):
        """Returns an iterator over the keys.
        """
        return self._session.iterkeys()
    
    def itervalues(self):
        """Returns an iterator over the values.
        """
        return self._session.itervalues()
    
    def iteritems(self):
        """Returns an iterator over the ``(key, value)`` items.
        """
        return self._session.iteritems()
    
    def get(self, key, default=None):
        """Gets the value under the given *key*. If the *key* doesn't exists,
        the *default* value is returned instead.
        """
        return self._session.get(key, default)
    
    def pop(self, key, default=None):
        """Removes the specified *key* and returns the corresponding value.
        If *key* is not found, *default* is returned instead.
        """
        value = self._session.pop(key, default)
        if key in self._used:
            del self._used[key]
        return value
    
    def has_key(self, key):
        """Returns ``True`` if there's a value under the given *key*.
        
        .. deprecated:: 1.4.2
           :meth:`has_key()` is deprecated in favor of ``key in f``.
        """
        return self._session.has_key(key)
    
    def put(self, **kwargs):
        """Puts one or more values into this flash scope.
        """
        for key, value in kwargs.items():
            self[key] = value
    
    def clear(self):
        """Removes all items from this flash scope.
        """
        self._session.clear()
        self._used.clear()
    
    def put_immediate(self, key, value):
        """Puts a value inside this flash scope and marks it as *used*.
        """
        self._session[key] = value
        self._update_status(key)
    
    def discard(self, *keys):
        """Marks the entire current flash or a single value as *used*, so when
        the next request hit the server, those values will be automatically
        removed from this flash scope by :class:`FlashMiddleware`.
        """
        self._update_status(*keys)
    
    def keep(self, *keys):
        """Prevents specific values from being removed on the next request.
        If this method is called with no args, the entire flash is preserved.
        """
        if not keys:
            self._update_status(is_used=False)
        else:
            for key in keys:
                self._update_status(key, is_used=False)
    
    def update(self):
        """Mark for removal entries that were kept, and delete unkept ones.
        
        .. note::
           This method is called automatically by
           :class:`djangoflash.middleware.FlashMiddleware` when a HTTP
           request hits the server, so never call this method yourself,
           unless you have a very good reason to do so.
        """
        self._update_status()


class _ImmediateFlashScopeAdapter(object):
    """This class is used to add support for immediate flash values to an
    existing instance of :class:`FlashScope`. An immediate flash value is a
    value that is available to this request, but not to the next.
    """
    
    def __init__(self, delegate):
        """Returns a new :class:`_ImmediateFlashScopeAdapter` object which
        delegates certain calls to the given *delegate*.
        """
        self.delegate = delegate
    
    def __getitem__(self, key):
        """Retrieves a value. Raises a :exc:`KeyError` if *key* does
        not exists.
        """
        return self.delegate[key]
    
    def __contains__(self, key):
        """Returns ``True`` if there's a value under the given *key*.
        """
        return key in self.delegate
    
    def __setitem__(self, key, value):
        """Puts a *value* that *won't* be available to the next request, only to
        the current.
        """
        self.delegate.put_immediate(key, value)
    
    def __call__(self, **kwargs):
        """Puts values that *won't* be available to the next request, only to
        the current.
        """
        for key, value in kwargs.items():
            self[key] = value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import ez_setup
ez_setup.use_setuptools()

from setuptools import setup, find_packages

setup(
    name = 'django-flash',
    version = '1.4.2',
    author = 'Daniel Fernandes Martins',
    author_email = 'daniel.tritone@gmail.com',
    description = 'Rails-like flash scope support for Django.',
    license = 'LGPL',
    platforms = ['Any'],
    keywords = ['django', 'flash', 'session', 'scope', 'contrib'],
    url = 'http://djangoflash.destaquenet.com/',
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Topic :: Internet :: WWW/HTTP'
    ],
    install_requires = ['Django>=1.0_final'],
    packages = find_packages('src'),
    package_dir = {'':'src'},
    include_package_data=True,
    zip_safe = False,
    test_suite = 'djangoflash.tests.suite',
)




# Django settings for testproj project.

DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = 'db'

SECRET_KEY = 'g9b@q$)=^xd2g@-7pg=j=h3*8+xd#hgn-9je@iq5_m#seg&d1y'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'djangoflash.middleware.FlashMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'djangoflash.context_processors.flash',
)

ROOT_URLCONF = 'testproj.urls'

import os

TEMPLATE_DIRS = (
    os.path.normpath(os.path.dirname(__file__) + '/templates'),
)

INSTALLED_APPS = (
    'app',
    'django.contrib.sessions',
)
